<?php

/**
 * The admin-specific functionality of the plugin.
 *
 * @link       https://www.simonebaldini.dev
 * @since      1.0.0
 *
 * @package    Enhanced_Search
 * @subpackage Enhanced_Search/admin
 */

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    Enhanced_Search
 * @subpackage Enhanced_Search/admin
 * @author     Simone Baldini <info@simonebaldini.dev>
 */
class Enhanced_Search_Admin extends Enhanced_Search_Settings {

	const OPTION_ACTIVATION_TIME = 'enanched_search_activation_timestamp';
	const OPTION_REINDEX_TIME    = 'enanched_search_reindex_timestamp';

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/**
	 * The Lucene index.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      \ZendSearch\Lucene\SearchIndexInterface    $index
	 */
	private $index;

	/**
	 * WordPress admin plugin menus
	 *
	 * @var array
	 */
	private $menus = [];

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string                                  $plugin_name       The name of this plugin.
	 * @param      string                                  $version    The version of this plugin.
	 * @param      \ZendSearch\Lucene\SearchIndexInterface $index The Lucene index.
	 */
	public function __construct( $plugin_name, $version, $index ) {

		$this->plugin_name = $plugin_name;
		$this->version     = $version;
		$this->index       = $index;

	}

	/**
	 * Register the stylesheets for the admin area.
	 *
	 * @since 1.0.0
	 */
	public function enqueue_styles() {
		$screen = get_current_screen();
		switch ( $screen->id ) {
			case $this->menus['settings']:
				wp_enqueue_style(
					"{$this->plugin_name}-rangeslider",
					plugin_dir_url( __FILE__ ) . 'css/rangeslider.css',
					[],
					$this->version,
					'all'
				);
				wp_enqueue_style(
					$this->plugin_name,
					plugin_dir_url( __FILE__ ) . 'css/search-admin.css',
					[],
					$this->version,
					'all'
				);
				break;
		}
	}

	/**
	 * Register the JavaScript for the admin area.
	 *
	 * @since 1.0.0
	 */
	public function enqueue_scripts() {
		$screen = get_current_screen();
		switch ( $screen->id ) {
			case $this->menus['settings']:
				wp_enqueue_script(
					"{$this->plugin_name}-rangeslider",
					plugin_dir_url( __FILE__ ) . 'js/rangeslider.min.js',
					[ 'jquery' ],
					$this->version,
					false
				);
				wp_enqueue_script(
					$this->plugin_name,
					plugin_dir_url( __FILE__ ) . 'js/search-admin.js',
					[ 'jquery' ],
					$this->version,
					false
				);
				wp_localize_script(
					$this->plugin_name,
					'data',
					[
						'ajax_url' => admin_url( 'admin-ajax.php' ),
						'post_ids' => array_map(
							function ( $post ) {
								return $post->ID;
							},
							get_posts(
								[
									'numberposts' => -1,
									'post_status' => 'publish',
									'post_type'   => 'any',
								]
							)
						),
					]
				);
				break;
		}
	}

	/**
	 * Add WordPress admin menu.
	 *
	 * @since 1.0.0
	 * @return void
	 */
	public function add_menu() {
		$this->menus['settings'] = add_submenu_page(
			'tools.php',
			__(
				'Enhanced Search',
				'enhanced_search'
			),
			__(
				'Enhanced Search',
				'enhanced_search'
			),
			'administrator',
			'enhanced_search',
			[
				$this,
				'render',
			]
		);
	}

	/**
	 * Render WordPress admin settings page.
	 *
	 * @since 1.0.0
	 * @return void
	 */
	public function render() {
		require implode(
			DIRECTORY_SEPARATOR,
			[
				__DIR__,
				'partials',
				'enhanced-search-admin-display.php',
			]
		);
	}

	/**
	 * Register plugin's settings.
	 *
	 * @since 1.0.0
	 * @return void
	 */
	public function register_settings() {
		register_setting(
			'enhanced_search',
			'enhanced_search'
		);

		if (
			isset( $_POST['_wpnonce'], $_POST['enhanced_search'] ) &&
			wp_verify_nonce( sanitize_key( $_POST['_wpnonce'] ), '_enhanced_search_nonce' )
		) {
			$data = filter_input_array(
				INPUT_POST,
				[
					'enhanced_search' => [
						'filter' => \FILTER_SANITIZE_STRING,
						'flags'  => \FILTER_REQUIRE_ARRAY,
					],
				]
			);
			update_option( 'enhanced_search', $data['enhanced_search'] );
		}
	}

	/**
	 * Post trash hook.
	 *
	 * @param int $post_id The post ID.
	 * @return void
	 */
	public function trashed_post( $post_id ) {
		$term  = new \ZendSearch\Lucene\Index\Term( $post_id, 'post_id' );
		$query = new \ZendSearch\Lucene\Search\Query\Term( $term );
		$hits  = $this->index->find( $query );
		foreach ( $hits as $hit ) {
			$this->index->delete( $hit->id );
		}
	}

	/**
	 * Post save hook.
	 *
	 * @param int     $post_id  The post ID.
	 * @param WP_Post $post The post.
	 * @since 1.0.0
	 * @return void
	 */
	public function save_post( $post_id, $post ) {
		if ( 'publish' === $post->post_status ) {
			$term  = new \ZendSearch\Lucene\Index\Term( $post_id, 'post_id' );
			$query = new \ZendSearch\Lucene\Search\Query\Term( $term );
			$hits  = $this->index->find( $query );
			foreach ( $hits as $hit ) {
				$this->index->delete( $hit->id );
			}
			$doc = new \ZendSearch\Lucene\Document();
			$doc->addField( \ZendSearch\Lucene\Document\Field::keyword( 'post_id', $post_id ) );
			$doc->addField( \ZendSearch\Lucene\Document\Field::Text( 'post_type', wp_strip_all_tags( $post->post_type ) ) );
			$doc->addField( \ZendSearch\Lucene\Document\Field::Text( 'post_title', wp_strip_all_tags( $post->post_title ) ) );
			$doc->addField( \ZendSearch\Lucene\Document\Field::Text( 'post_excerpt', wp_strip_all_tags( $post->post_excerpt ) ) );
			$doc->addField( \ZendSearch\Lucene\Document\Field::Text( 'post_content', wp_strip_all_tags( $post->post_content ) ) );
			$taxonomies = get_post_taxonomies( $post );
			foreach ( $taxonomies as $taxonomy ) {
				$terms = array_map(
					function ( $term ) {
						return $term->name;
					},
					get_the_terms( $post, $taxonomy ) ?: []
				);
				$doc->addField( \ZendSearch\Lucene\Document\Field::Text( $taxonomy, implode( ' ', $terms ) ) );
			}
			$this->index->addDocument( $doc );
		}
	}

	/**
	 * Reindex a post.
	 *
	 * @since 1.0.0
	 * @return void
	 */
	public function reindex() {
		$start   = round( microtime( true ) * 1000 );
		$post_id = filter_input( INPUT_POST, 'post_id', FILTER_VALIDATE_INT );
		$last    = filter_input( INPUT_POST, 'last', FILTER_VALIDATE_BOOLEAN );
		$post    = get_post( $post_id );
		$this->save_post( $post_id, $post );
		$end = round( microtime( true ) * 1000 );
		if ( $last ) {
			update_option( self::OPTION_REINDEX_TIME, time() );
		}
		wp_send_json_success(
			[
				time(),
				$post->ID,
				$post->post_type,
				$post->post_title,
				$end - $start,
			]
		);
	}

	/**
	 * Undocumented function
	 *
	 * @return void
	 */
	public function admin_notices() {
		$activation = get_option( self::OPTION_ACTIVATION_TIME, false );
		$reindex    = get_option( self::OPTION_REINDEX_TIME, false );
		if ( ! $reindex || $activation >= $reindex ) {
			$message = sprintf(
				__( 'You may need to <a href="%s">reindex your posts</a>.', 'enhanced_search' ),
				menu_page_url( 'enhanced_search', false )
			);
			echo sprintf(
				'<div class="notice notice-warning is-dismissible">
					<p>%s</p>
				</div>',
				$message
			);
		}
	}

}
