<?php

/**
 * The file that defines the core plugin class
 *
 * A class definition that includes attributes and functions used across both the
 * public-facing side of the site and the admin area.
 *
 * @link       https://www.simonebaldini.dev
 * @since      1.0.0
 *
 * @package    Enhanced_Search
 * @subpackage Enhanced_Search/includes
 */

/**
 * The core plugin class.
 *
 * This is used to define internationalization, admin-specific hooks, and
 * public-facing site hooks.
 *
 * Also maintains the unique identifier of this plugin as well as the current
 * version of the plugin.
 *
 * @since      1.0.0
 * @package    Enhanced_Search
 * @subpackage Enhanced_Search/includes
 * @author     Simone Baldini <info@simonebaldini.dev>
 */
class Enhanced_Search extends Enhanced_Search_Settings {

	/**
	 * The loader that's responsible for maintaining and registering all hooks that power
	 * the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      Enhanced_Search_Loader    $loader    Maintains and registers all hooks for the plugin.
	 */
	protected $loader;

	/**
	 * The unique identifier of this plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $plugin_name    The string used to uniquely identify this plugin.
	 */
	protected $plugin_name;

	/**
	 * The current version of the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $version    The current version of the plugin.
	 */
	protected $version;

	/**
	 * Define the core functionality of the plugin.
	 *
	 * Set the plugin name and the plugin version that can be used throughout the plugin.
	 * Load the dependencies, define the locale, and set the hooks for the admin area and
	 * the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function __construct() {
		if ( defined( 'ENHANCED_SEARCH_VERSION' ) ) {
			$this->version = ENHANCED_SEARCH_VERSION;
		} else {
			$this->version = '1.0.0';
		}
		$this->plugin_name = 'enhanced-search';

		$this->load_dependencies();
		$this->set_locale();
		$this->define_admin_hooks();
		$this->define_public_hooks();

	}

	/**
	 * Load the required dependencies for this plugin.
	 *
	 * Include the following files that make up the plugin:
	 *
	 * - Enhanced_Search_Loader. Orchestrates the hooks of the plugin.
	 * - Enhanced_Search_i18n. Defines internationalization functionality.
	 * - Enhanced_Search_Admin. Defines all hooks for the admin area.
	 * - Enhanced_Search_Public. Defines all hooks for the public side of the site.
	 *
	 * Create an instance of the loader which will be used to register the hooks
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function load_dependencies() {

		/**
		 * The class responsible for orchestrating the actions and filters of the
		 * core plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-enhanced-search-loader.php';

		/**
		 * The class responsible for defining internationalization functionality
		 * of the plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-enhanced-search-i18n.php';

		/**
		 * The class responsible for defining all actions that occur in the admin area.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-enhanced-search-admin.php';

		/**
		 * The class responsible for defining all actions that occur in the public-facing
		 * side of the site.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'public/class-enhanced-search-public.php';

		$this->loader = new Enhanced_Search_Loader();

		spl_autoload_register(
			function ( $class ) {
				if ( strpos( $class, 'ZendSearch' ) === 0 ) {
					$dir  = dirname( __DIR__ );
					$path = implode( '/', explode( '\\', $class ) );
					require_once "{$dir}/library/{$path}.php";
				}
			}
		);

	}

	/**
	 * Define the locale for this plugin for internationalization.
	 *
	 * Uses the Enhanced_Search_i18n class in order to set the domain and to register the hook
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function set_locale() {

		$plugin_i18n = new Enhanced_Search_i18n();

		$this->loader->add_action( 'plugins_loaded', $plugin_i18n, 'load_plugin_textdomain' );

	}

	/**
	 * Register all of the hooks related to the admin area functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_admin_hooks() {

		$plugin_admin = new Enhanced_Search_Admin(
			$this->get_plugin_name(),
			$this->get_version(),
			$this->open_lucene_index()
		);

		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_styles' );
		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_scripts' );
		$this->loader->add_action( 'admin_menu', $plugin_admin, 'add_menu' );
		$this->loader->add_action( 'admin_init', $plugin_admin, 'register_settings' );
		$this->loader->add_action( 'trashed_post', $plugin_admin, 'trashed_post', PHP_INT_MAX );
		$this->loader->add_action( 'save_post', $plugin_admin, 'save_post', PHP_INT_MAX, 2 );
		$this->loader->add_action( 'wp_ajax_reindex', $plugin_admin, 'reindex' );
		$this->loader->add_action( 'wp_ajax_optimize', $plugin_admin, 'optimize' );
		$this->loader->add_action( 'wp_ajax_datatable', $plugin_admin, 'datatable' );
		$this->loader->add_action( 'admin_notices', $plugin_admin, 'admin_notices' );

	}

	/**
	 * Register all of the hooks related to the public-facing functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_public_hooks() {

		$plugin_public = new Enhanced_Search_Public(
			$this->get_plugin_name(),
			$this->get_version(),
			$this->open_lucene_index()
		);

		$this->loader->add_action( 'posts_pre_query', $plugin_public, 'posts_pre_query', PHP_INT_MAX, 2 );

	}

	/**
	 * Run the loader to execute all of the hooks with WordPress.
	 *
	 * @since    1.0.0
	 */
	public function run() {
		$this->loader->run();
	}

	/**
	 * The name of the plugin used to uniquely identify it within the context of
	 * WordPress and to define internationalization functionality.
	 *
	 * @since     1.0.0
	 * @return    string    The name of the plugin.
	 */
	public function get_plugin_name() {
		return $this->plugin_name;
	}

	/**
	 * The reference to the class that orchestrates the hooks with the plugin.
	 *
	 * @since     1.0.0
	 * @return    Enhanced_Search_Loader    Orchestrates the hooks of the plugin.
	 */
	public function get_loader() {
		return $this->loader;
	}

	/**
	 * Retrieve the version number of the plugin.
	 *
	 * @since     1.0.0
	 * @return    string    The version number of the plugin.
	 */
	public function get_version() {
		return $this->version;
	}

	/**
	 * Open or create the index.
	 *
	 * @since  1.0.0
	 * @return \ZendSearch\Lucene\SearchIndexInterface
	 */
	private function open_lucene_index() {
		$upload_dir = wp_upload_dir();
		$path       = sprintf(
			'%s/enhanced-search/%d',
			$upload_dir['basedir'],
			get_current_blog_id()
		);
		$index      = file_exists( $path ) && is_dir( $path )
			? \ZendSearch\Lucene\Lucene::open( $path )
			: \ZendSearch\Lucene\Lucene::create( $path );
		// $index->setMaxBufferedDocs(1);
		// $index->setMergeFactor(3);
		return $index;
	}

}
