<?php

/**
 * The public-facing functionality of the plugin.
 *
 * @link       https://www.simonebaldini.dev
 * @since      1.0.0
 *
 * @package    Enhanced_Search
 * @subpackage Enhanced_Search/public
 */

/**
 * The public-facing functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the public-facing stylesheet and JavaScript.
 *
 * @package    Enhanced_Search
 * @subpackage Enhanced_Search/public
 * @author     Simone Baldini <info@simonebaldini.dev>
 */
class Enhanced_Search_Public extends Enhanced_Search_Settings {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/**
	 * The Lucene index.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      \ZendSearch\Lucene\SearchIndexInterface    $index
	 */
	private $index;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string                                  $plugin_name       The name of the plugin.
	 * @param      string                                  $version    The version of this plugin.
	 * @param      \ZendSearch\Lucene\SearchIndexInterface $index The Lucene index.
	 */
	public function __construct( $plugin_name, $version, $index ) {

		$this->plugin_name = $plugin_name;
		$this->version     = $version;
		$this->index       = $index;

	}

	/**
	 * Alter posts results only on search.
	 *
	 * @param array    $posts The matched posts.
	 * @param WP_Query $wp_query The current query.
	 * @since  1.0.0
	 * @return array
	 */
	public function posts_pre_query( $posts, $wp_query ) {
		if ( is_search() && true === $wp_query->is_search ) {
			$posts               = [];
			$query               = new \ZendSearch\Lucene\Search\Query\Boolean();
			$options_post_types  = array_filter(
				$this->get_option( 'post_types', [] ),
				function ( $settings ) {
					return array_key_exists( 'active', $settings ) && boolval( $settings['active'] );
				}
			);
			$post_types_subquery = new \ZendSearch\Lucene\Search\Query\Boolean();
			foreach ( $options_post_types as $name => $settings ) {
				$term     = new \ZendSearch\Lucene\Index\Term( $name, 'post_type' );
				$subquery = new \ZendSearch\Lucene\Search\Query\Term( $term );
				$subquery->setBoost( pow( $settings['boost'], 20 ) );
				$post_types_subquery->addSubquery( $subquery );
			}
			if ( ! empty( $post_types_subquery->getSubqueries() ) ) {
				$query->addSubquery( $post_types_subquery, true );
			}
			$options_attributes = array_filter(
				$this->get_option( 'attributes', [] ),
				function ( $settings ) {
					return array_key_exists( 'active', $settings ) && boolval( $settings['active'] );
				}
			);
			foreach ( $options_attributes as $name => $settings ) {
				$subquery_boosted = new \ZendSearch\Lucene\Search\Query\Boolean();
				$subquery_boosted->setBoost( $settings['boost'] );
				$term     = new \ZendSearch\Lucene\Index\Term( get_search_query(), $name );
				$subquery = $settings['sensibility'] < 1
					? new \ZendSearch\Lucene\Search\Query\Fuzzy( $term, $settings['sensibility'], 0 )
					: new \ZendSearch\Lucene\Search\Query\Term( $term );
				$subquery_boosted->addSubquery( $subquery );
				$query->addSubquery( $subquery_boosted );
			}
			$options_taxonomies = array_filter(
				$this->get_option( 'taxonomies', [] ),
				function ( $settings ) {
					return array_key_exists( 'active', $settings ) && boolval( $settings['active'] );
				}
			);
			foreach ( $options_taxonomies as $name => $settings ) {
				$subquery_boosted = new \ZendSearch\Lucene\Search\Query\Boolean();
				$subquery_boosted->setBoost( $settings['boost'] );
				$term     = new \ZendSearch\Lucene\Index\Term( get_search_query(), $name );
				$subquery = $settings['sensibility'] < 1
					? new \ZendSearch\Lucene\Search\Query\Fuzzy( $term, $settings['sensibility'], 0 )
					: new \ZendSearch\Lucene\Search\Query\Term( $term );
				$subquery_boosted->addSubquery( $subquery );
				$query->addSubquery( $subquery_boosted );
			}

			$hits = array_filter(
				$this->index->find( $query ),
				function ( $hit ) {
					return $hit->score >= 0.5;
				}
			);

			foreach ( $hits as $hit ) {
				$posts[] = get_post( $hit->post_id );
			}

			return $posts;
		}
	}
}
